#include <iostream>

#include "./common.h"
#include "./types.h"
#include "./geom.h"
#include "./vec.h"

using namespace std;

Vec3f red(1, 0, 0);
Vec3f green(0, 0, 1);
Vec3f blue(0, 0, 1);

void DrawTriangle(GLfloat z, const Vec4f& color) {
  glColor4fv(color);
  glBegin(GL_TRIANGLES);
  glVertex3f(-10, 0, z);
  glVertex3f(10, 0, z);
  glVertex3f(0, 15, z);
  glEnd();
}

void DrawTriangle(GLfloat z, const Vec3f& color, GLfloat a) {
  DrawTriangle(z, Vec4f(color, a));
}

void DrawTriangle(GLfloat z, GLfloat r, GLfloat g, GLfloat b, GLfloat a) {
  DrawTriangle(z, Vec4f(r, g, b, a));
}

void Display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  DrawAxis();

  DrawTriangle(10, red, 1);
  DrawTriangle(-10, blue, 1);

  glFlush();
  glutSwapBuffers();
}

void Display1() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  // red at z = 10
  DrawTriangle(10, red, 1);
  // blue at z = -10
  DrawTriangle(-10, blue, 1);

  glFlush();
  glutSwapBuffers();
}

void Display2() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // this line added
  glDisable(GL_DEPTH_TEST);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  DrawTriangle(10, red, 1);
  DrawTriangle(-10, blue, 1);

  glFlush();
  glutSwapBuffers();
}

void Display3() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  // these lines changed
  DrawTriangle(-10, blue, 1);
  DrawTriangle(10, red, 0.5);

  glFlush();
  glutSwapBuffers();
}

void Display4() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_BLEND);
  // glBlendFunc(GL_ONE, GL_ZERO);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  // glBlendFunc(GL_ONE_MINUS_DST_ALPHA, GL_DST_ALPHA);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  // these lines switched
  DrawTriangle(10, red, 0.5);
  DrawTriangle(-10, blue, 1);

  glFlush();
  glutSwapBuffers();
}

void Display5() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_BLEND);
  // blending function changed
  glBlendFunc(GL_ONE, GL_ZERO);
  // glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  // glBlendFunc(GL_ONE_MINUS_DST_ALPHA, GL_DST_ALPHA);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  // these lines switched back
  DrawTriangle(-10, blue, 1);
  DrawTriangle(10, red, 0.5);

  glFlush();
  glutSwapBuffers();
}

void Display6() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_BLEND);
  // blending function changed
  glBlendFunc(GL_ZERO, GL_ONE);
  // glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  // glBlendFunc(GL_ONE_MINUS_DST_ALPHA, GL_DST_ALPHA);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  DrawTriangle(-10, blue, 1);
  DrawTriangle(10, red, 0.5);

  glFlush();
  glutSwapBuffers();
}

void Display7() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_BLEND);
  // blending function changed
  // note that color buffer clear color is (1, 1, 1, 1)
  glBlendFunc(GL_ONE_MINUS_DST_ALPHA, GL_DST_ALPHA);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  DrawTriangle(-10, blue, 1);
  DrawTriangle(10, red, 0.5);

  glFlush();
  glutSwapBuffers();
}

void Display8() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_BLEND);
  // blending function changed
  glBlendFunc(GL_DST_ALPHA, GL_ONE_MINUS_DST_ALPHA);

  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  DrawTriangle(-10, blue, 1);
  DrawTriangle(10, red, 0.5);

  glFlush();
  glutSwapBuffers();
}

void Display9() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Use Mac screen zoom to see aliasing effects.
  // To enable: system properties > universal access
  // Toggle smoothing off: Command+Option+\
  // To zoom: Command+Option+= to zoom in
  //          Command+Option+- to zoom out
  glLineWidth(2.0);
  DrawAxis();

  glFlush();
  glutSwapBuffers();
}

void Display10() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Heads up - this might not be supported by your card!
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glLineWidth(2.0);
  DrawAxis();

  glFlush();
  glutSwapBuffers();
}

void Display11() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_POLYGON_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glLineWidth(2.0);
  DrawAxis();

  // What is drawn?
  //  A) solid red triangle in front of blue
  //  B) solid blue triangle in front of red
  //  C) translucent red in front of blue
  //  D) none of the above

  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  DrawTriangle(-10, blue, 1);
  DrawTriangle(10, red, 1);

  glFlush();
  glutSwapBuffers();
}

void Display12() {
}

void Display13() {
}
