#include <iostream>
#include <iomanip>

#include "./common.h"
#include "./types.h"
#include "./geom.h"

using namespace std;

// window parameters
int window_width = 800, window_height = 600;
float window_aspect = (float)window_width / (float)window_height;

#define PI 3.14159

void Display();
void Display1();
void Display2();
void Display3();
void Display4();
void Display5();
void Display6();
void Display7();
void Display8();
void Display9();
void Display10();
void Display11();
void Display12();
void Display13();

void PrintMatrix() {
  GLfloat m[16];
  glGetFloatv(GL_MODELVIEW_MATRIX, m);
  cout.precision(2);
  int w = 6;
  for (int i = 0; i < 4; ++i) {
    cout << setprecision(2) << setw(w) << m[i] << " " 
         << setprecision(2) << setw(w) << m[i+4] << " "
         << setprecision(2) << setw(w) << m[i+8] << " "
         << setprecision(2) << setw(w) << m[i+12] << " "
         << endl;
  }
  cout << endl;
}

void LoadMatrix(GLfloat* m) {
  // transpose to column-major
  for (int i = 0; i < 4; ++i) {
    for (int j = i; j < 4; ++j) {
      swap(m[i*4+j], m[j*4+i]);
    }
  }
  glLoadMatrixf(m);
}

void MultMatrix(GLfloat* m) {
  // transpose to column-major
  for (int i = 0; i < 4; ++i) {
    for (int j = i; j < 4; ++j) {
      swap(m[i*4+j], m[j*4+i]);
    }
  }
  glMultMatrixf(m);
}

void Init() {
  glEnable(GL_DEPTH_TEST);
  glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

  // resize the window
  window_aspect = window_width/(float)window_height;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-15, 15, -5, 20, 0, 200);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(5, 5, 45,
            0, 0, 0,
            0, 1, 0);

  DisableLighting();
}

void DisableLighting() {
  glShadeModel(GL_FLAT);
  glDisable(GL_LIGHTING);
  glDisable(GL_LIGHT0);
  glDisable(GL_COLOR_MATERIAL);
}

void EnableLighting() {
  glShadeModel(GL_SMOOTH);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_COLOR_MATERIAL);

  GLfloat position[] = { -150, 150, 300, 1 };
  glLightfv(GL_LIGHT0, GL_POSITION, position);

  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
}

void DrawAxis() {
  DisableLighting();

  const Vec3f c(0, 0, 0);
  const float L = 20;
  const Vec3f X(L, 0, 0), Y(0, L, 0), Z(0, 0, L);
		
  // glLineWidth(2.0);

  glBegin(GL_LINES);
	
  glColor3f(1,0,0);
  glVertex3fv(c.x);
  glVertex3fv((c+X).x);
	
  glColor3f(0,1,0);
  glVertex3fv(c.x);
  glVertex3fv((c+Y).x);
	
  glColor3f(0,0,1);
  glVertex3fv(c.x);
  glVertex3fv((c+Z).x);
	
  glEnd();
}

void Keyboard(unsigned char key, int x, int y) {
  switch (key) {
    case 'q':
    case 27: // esc
      exit(0);
      break;
  }
}

int main(int argc, char *argv[]) {
  if (argc < 2) {
    cout << "Usage: test <test_num>" << endl;
    return 0;
  }

  // Initialize GLUT
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_ALPHA);
  // glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_ALPHA);
  glutInitWindowSize(window_width, window_height);
  glutInitWindowPosition(100, 100);
  glutCreateWindow("test");
  glutKeyboardFunc(Keyboard);
  // glutDisplayFunc(Display);

  const int i = atoi(argv[1]);
  if (i == 1) glutDisplayFunc(Display1);
  else if (i == 2) glutDisplayFunc(Display2);
  else if (i == 3) glutDisplayFunc(Display3);
  else if (i == 4) glutDisplayFunc(Display4);
  else if (i == 5) glutDisplayFunc(Display5);
  else if (i == 6) glutDisplayFunc(Display6);
  else if (i == 7) glutDisplayFunc(Display7);
  else if (i == 8) glutDisplayFunc(Display8);
  else if (i == 9) glutDisplayFunc(Display9);
  else if (i == 10) glutDisplayFunc(Display10);
  else if (i == 11) glutDisplayFunc(Display11);
  else if (i == 12) glutDisplayFunc(Display12);
  else if (i == 13) glutDisplayFunc(Display13);
  // else if (i == 14) glutDisplayFunc(Display14);
  // else if (i == 15) glutDisplayFunc(Display15);
  // else if (i == 16) glutDisplayFunc(Display16);
  // else if (i == 17) glutDisplayFunc(Display17);
  // else if (i == 18) glutDisplayFunc(Display18);
  // else if (i == 19) glutDisplayFunc(Display19);
  // else if (i == 20) glutDisplayFunc(Display20);
  // else if (i == 21) glutDisplayFunc(Display21);
  else glutDisplayFunc(Display);

  Init();

  glutMainLoop();

  return 0;
}
